@@ -1,13 +1,12 @@
 use std::borrow::Cow;
 
 use itertools::Itertools;
-
 use ruff_macros::{ViolationMetadata, derive_message_formats};
-use ruff_python_ast::str::{leading_quote, trailing_quote};
+use ruff_python_ast::StringFlags;
 use ruff_python_index::Indexer;
-use ruff_python_parser::{TokenKind, Tokens};
+use ruff_python_parser::{Token, TokenKind, Tokens};
 use ruff_source_file::LineRanges;
-use ruff_text_size::{Ranged, TextRange};
+use ruff_text_size::{Ranged, TextLen, TextRange};
 
 use crate::Locator;
 use crate::checkers::ast::LintContext;
@@ -169,7 +168,8 @@ pub(crate) fn implicit(
                 SingleLineImplicitStringConcatenation,
                 TextRange::new(a_range.start(), b_range.end()),
             ) {
-                if let Some(fix) = concatenate_strings(a_range, b_range, locator) {
+                if let Some(fix) = concatenate_strings(a_token, b_token, a_range, b_range, locator)
+                {
                     diagnostic.set_fix(fix);
                 }
             }
@@ -177,38 +177,55 @@ pub(crate) fn implicit(
     }
 }
 
-fn concatenate_strings(a_range: TextRange, b_range: TextRange, locator: &Locator) -> Option<Fix> {
-    let a_text = locator.slice(a_range);
-    let b_text = locator.slice(b_range);
-
-    let a_leading_quote = leading_quote(a_text)?;
-    let b_leading_quote = leading_quote(b_text)?;
-
-    // Require, for now, that the leading quotes are the same.
-    if a_leading_quote != b_leading_quote {
+/// Concatenates two strings
+///
+/// The `a_string_range` and `b_string_range` are the range of the entire string,
+/// not just of the string token itself (important for interpolated strings where
+/// the start token doesn't span the entire token).
+fn concatenate_strings(
+    a_token: &Token,
+    b_token: &Token,
+    a_string_range: TextRange,
+    b_string_range: TextRange,
+    locator: &Locator,
+) -> Option<Fix> {
+    if a_token.string_flags()?.is_unclosed() || b_token.string_flags()?.is_unclosed() {
         return None;
     }
 
-    let a_trailing_quote = trailing_quote(a_text)?;
-    let b_trailing_quote = trailing_quote(b_text)?;
+    let a_string_flags = a_token.string_flags()?;
+    let b_string_flags = b_token.string_flags()?;
 
-    // Require, for now, that the trailing quotes are the same.
-    if a_trailing_quote != b_trailing_quote {
+    let a_prefix = a_string_flags.prefix();
+    let b_prefix = b_string_flags.prefix();
+
+    // Require, for now, that the strings have the same prefix,
+    // quote style, and number of quotes
+    if a_prefix != b_prefix
+        || a_string_flags.quote_style() != b_string_flags.quote_style()
+        || a_string_flags.is_triple_quoted() != b_string_flags.is_triple_quoted()
+    {
         return None;
     }
 
+    let a_text = locator.slice(a_string_range);
+    let b_text = locator.slice(b_string_range);
+
+    let quotes = a_string_flags.quote_str();
+
+    let opener_len = a_string_flags.opener_len();
+    let closer_len = a_string_flags.closer_len();
+
     let mut a_body =
-        Cow::Borrowed(&a_text[a_leading_quote.len()..a_text.len() - a_trailing_quote.len()]);
-    let b_body = &b_text[b_leading_quote.len()..b_text.len() - b_trailing_quote.len()];
+        Cow::Borrowed(&a_text[TextRange::new(opener_len, a_text.text_len() - closer_len)]);
+    let b_body = &b_text[TextRange::new(opener_len, b_text.text_len() - closer_len)];
 
-    if a_leading_quote.find(['r', 'R']).is_none()
-        && matches!(b_body.bytes().next(), Some(b'0'..=b'7'))
-    {
+    if !a_string_flags.is_raw_string() && matches!(b_body.bytes().next(), Some(b'0'..=b'7')) {
         normalize_ending_octal(&mut a_body);
     }
 
-    let concatenation = format!("{a_leading_quote}{a_body}{b_body}{a_trailing_quote}");
-    let range = TextRange::new(a_range.start(), b_range.end());
+    let concatenation = format!("{a_prefix}{quotes}{a_body}{b_body}{quotes}");
+    let range = TextRange::new(a_string_range.start(), b_string_range.end());
 
     Some(Fix::safe_edit(Edit::range_replacement(
         concatenation,
