@@ -41,22 +41,24 @@ fn test_quiet_output() -> anyhow::Result<()> {
     let case = CliTest::with_file("test.py", "x: int = 'foo'")?;
 
     // By default, we emit a diagnostic
-    assert_cmd_snapshot!(case.command(), @r###"
+    assert_cmd_snapshot!(case.command(), @r#"
     success: false
     exit_code: 1
     ----- stdout -----
     error[invalid-assignment]: Object of type `Literal["foo"]` is not assignable to `int`
-     --> test.py:1:1
+     --> test.py:1:4
       |
     1 | x: int = 'foo'
-      | ^
+      |    ---   ^^^^^ Incompatible value of type `Literal["foo"]`
+      |    |
+      |    Declared type
       |
     info: rule `invalid-assignment` is enabled by default
 
     Found 1 diagnostic
 
     ----- stderr -----
-    "###);
+    "#);
 
     // With `quiet`, the diagnostic is not displayed, just the summary message
     assert_cmd_snapshot!(case.command().arg("--quiet"), @r"
@@ -560,9 +562,9 @@ fn check_non_existing_path() -> anyhow::Result<()> {
 
     assert_cmd_snapshot!(
         case.command().arg("project/main.py").arg("project/tests"),
-        @r###"
+        @r"
     success: false
-    exit_code: 1
+    exit_code: 2
     ----- stdout -----
     error[io]: `<temp_dir>/project/main.py`: No such file or directory (os error 2)
 
@@ -572,7 +574,7 @@ fn check_non_existing_path() -> anyhow::Result<()> {
 
     ----- stderr -----
     WARN No python files found under the given path(s)
-    "###
+    "
     );
 
     Ok(())
@@ -617,7 +619,7 @@ fn gitlab_diagnostics() -> anyhow::Result<()> {
     let _s = settings.bind_to_scope();
 
     assert_cmd_snapshot!(case.command().arg("--output-format=gitlab").arg("--warn").arg("unresolved-reference")
-        .env("CI_PROJECT_DIR", case.project_dir), @r###"
+        .env("CI_PROJECT_DIR", case.project_dir), @r#"
     success: false
     exit_code: 1
     ----- stdout -----
@@ -655,14 +657,14 @@ fn gitlab_diagnostics() -> anyhow::Result<()> {
             },
             "end": {
               "line": 3,
-              "column": 8
+              "column": 11
             }
           }
         }
       }
     ]
     ----- stderr -----
-    "###);
+    "#);
 
     Ok(())
 }
@@ -677,15 +679,15 @@ fn github_diagnostics() -> anyhow::Result<()> {
         "#,
     )?;
 
-    assert_cmd_snapshot!(case.command().arg("--output-format=github").arg("--warn").arg("unresolved-reference"), @r###"
+    assert_cmd_snapshot!(case.command().arg("--output-format=github").arg("--warn").arg("unresolved-reference"), @r"
     success: false
     exit_code: 1
     ----- stdout -----
     ::warning title=ty (unresolved-reference),file=<temp_dir>/test.py,line=2,col=7,endLine=2,endColumn=8::test.py:2:7: unresolved-reference: Name `x` used when not defined
-    ::error title=ty (non-subscriptable),file=<temp_dir>/test.py,line=3,col=7,endLine=3,endColumn=8::test.py:3:7: non-subscriptable: Cannot subscript object of type `Literal[4]` with no `__getitem__` method
+    ::error title=ty (non-subscriptable),file=<temp_dir>/test.py,line=3,col=7,endLine=3,endColumn=11::test.py:3:7: non-subscriptable: Cannot subscript object of type `Literal[4]` with no `__getitem__` method
 
     ----- stderr -----
-    "###);
+    ");
 
     Ok(())
 }
