@@ -61,6 +61,7 @@ pub(crate) enum GotoTarget<'a> {
     /// ```
     ImportModuleComponent {
         module_name: String,
+        level: u32,
         component_index: usize,
         component_range: TextRange,
     },
@@ -189,20 +190,31 @@ pub(crate) enum GotoTarget<'a> {
         /// The call of the callable
         call: &'a ast::ExprCall,
     },
+
+    /// Go to on a sub-expression of a string annotation's sub-AST
+    ///
+    /// ```py
+    /// x: "int | None"
+    ///           ^^^^
+    /// ```
+    ///
+    /// This is equivalent to `GotoTarget::Expression` but the expression
+    /// isn't actually in the AST.
+    StringAnnotationSubexpr {
+        /// The string literal that is a string annotation.
+        string_expr: &'a ast::ExprStringLiteral,
+        /// The range to query in the sub-AST for the sub-expression.
+        subrange: TextRange,
+        /// If the expression is a Name of some kind this is the name (just a cached result).
+        name: Option<String>,
+    },
 }
 
 /// The resolved definitions for a `GotoTarget`
 #[derive(Debug, Clone)]
-pub(crate) enum DefinitionsOrTargets<'db> {
-    /// We computed actual Definitions we can do followup queries on.
-    Definitions(Vec<ResolvedDefinition<'db>>),
-    /// We directly computed a navigation.
-    ///
-    /// We can't get docs or usefully compute goto-definition for this.
-    Targets(crate::NavigationTargets),
-}
+pub(crate) struct Definitions<'db>(pub Vec<ResolvedDefinition<'db>>);
 
-impl<'db> DefinitionsOrTargets<'db> {
+impl<'db> Definitions<'db> {
     pub(crate) fn from_ty(db: &'db dyn crate::Db, ty: Type<'db>) -> Option<Self> {
         let ty_def = ty.definition(db)?;
         let resolved = match ty_def {
@@ -218,7 +230,7 @@ impl<'db> DefinitionsOrTargets<'db> {
                 ResolvedDefinition::Definition(definition)
             }
         };
-        Some(DefinitionsOrTargets::Definitions(vec![resolved]))
+        Some(Definitions(vec![resolved]))
     }
 
     /// Get the "goto-declaration" interpretation of this definition
@@ -226,14 +238,9 @@ impl<'db> DefinitionsOrTargets<'db> {
     /// In this case it basically returns exactly what was found.
     pub(crate) fn declaration_targets(
         self,
-        db: &'db dyn crate::Db,
+        db: &'db dyn ty_python_semantic::Db,
     ) -> Option<crate::NavigationTargets> {
-        match self {
-            DefinitionsOrTargets::Definitions(definitions) => {
-                definitions_to_navigation_targets(db, None, definitions)
-            }
-            DefinitionsOrTargets::Targets(targets) => Some(targets),
-        }
+        definitions_to_navigation_targets(db, None, self.0)
     }
 
     /// Get the "goto-definition" interpretation of this definition
@@ -242,14 +249,9 @@ impl<'db> DefinitionsOrTargets<'db> {
     /// if the definition we have is found in a stub file.
     pub(crate) fn definition_targets(
         self,
-        db: &'db dyn crate::Db,
+        db: &'db dyn ty_python_semantic::Db,
     ) -> Option<crate::NavigationTargets> {
-        match self {
-            DefinitionsOrTargets::Definitions(definitions) => {
-                definitions_to_navigation_targets(db, Some(&StubMapper::new(db)), definitions)
-            }
-            DefinitionsOrTargets::Targets(targets) => Some(targets),
-        }
+        definitions_to_navigation_targets(db, Some(&StubMapper::new(db)), self.0)
     }
 
     /// Get the docstring for this definition
@@ -258,13 +260,7 @@ impl<'db> DefinitionsOrTargets<'db> {
     /// so this will check both the goto-declarations and goto-definitions (in that order)
     /// and return the first one found.
     pub(crate) fn docstring(self, db: &'db dyn crate::Db) -> Option<Docstring> {
-        let definitions = match self {
-            DefinitionsOrTargets::Definitions(definitions) => definitions,
-            // Can't find docs for these
-            // (make more cases DefinitionOrTargets::Definitions to get more docs!)
-            DefinitionsOrTargets::Targets(_) => return None,
-        };
-        for definition in &definitions {
+        for definition in &self.0 {
             // If we got a docstring from the original definition, use it
             if let Some(docstring) = definition.docstring(db) {
                 return Some(Docstring::new(docstring));
@@ -277,7 +273,7 @@ impl<'db> DefinitionsOrTargets<'db> {
         let stub_mapper = StubMapper::new(db);
 
         // Try to find the corresponding implementation definition
-        for definition in stub_mapper.map_definitions(definitions) {
+        for definition in stub_mapper.map_definitions(self.0) {
             if let Some(docstring) = definition.docstring(db) {
                 return Some(Docstring::new(docstring));
             }
@@ -302,26 +298,51 @@ impl GotoTarget<'_> {
             // (i.e. the type of `MyClass` in `MyClass()` is `<class MyClass>` and not `() -> MyClass`)
             GotoTarget::Call { callable, .. } => callable.inferred_type(model),
             GotoTarget::TypeParamTypeVarName(typevar) => typevar.inferred_type(model),
+            GotoTarget::ImportModuleComponent {
+                module_name,
+                component_index,
+                level,
+                ..
+            } => {
+                // We don't currently support hovering the bare `.` so there is always a name
+                let module = import_name(module_name, *component_index);
+                model.resolve_module_type(Some(module), *level)?
+            }
+            GotoTarget::StringAnnotationSubexpr {
+                string_expr,
+                subrange,
+                ..
+            } => {
+                let (subast, _submodel) = model.enter_string_annotation(string_expr)?;
+                let submod = subast.syntax();
+                let subnode = covering_node(submod.into(), *subrange).node();
+
+                // The type checker knows the type of the full annotation but nothing else
+                if AnyNodeRef::from(&*submod.body) == subnode {
+                    string_expr.inferred_type(model)
+                } else {
+                    // TODO: force the typechecker to tell us its secrets
+                    // (it computes but then immediately discards these types)
+                    return None;
+                }
+            }
+            GotoTarget::BinOp { expression, .. } => {
+                let (_, ty) = ty_python_semantic::definitions_for_bin_op(model, expression)?;
+                ty
+            }
+            GotoTarget::UnaryOp { expression, .. } => {
+                let (_, ty) = ty_python_semantic::definitions_for_unary_op(model, expression)?;
+                ty
+            }
             // TODO: Support identifier targets
             GotoTarget::PatternMatchRest(_)
             | GotoTarget::PatternKeywordArgument(_)
             | GotoTarget::PatternMatchStarName(_)
             | GotoTarget::PatternMatchAsName(_)
-            | GotoTarget::ImportModuleComponent { .. }
             | GotoTarget::TypeParamParamSpecName(_)
             | GotoTarget::TypeParamTypeVarTupleName(_)
             | GotoTarget::NonLocal { .. }
             | GotoTarget::Globals { .. } => return None,
-            GotoTarget::BinOp { expression, .. } => {
-                let (_, ty) =
-                    ty_python_semantic::definitions_for_bin_op(model.db(), model, expression)?;
-                ty
-            }
-            GotoTarget::UnaryOp { expression, .. } => {
-                let (_, ty) =
-                    ty_python_semantic::definitions_for_unary_op(model.db(), model, expression)?;
-                ty
-            }
         };
 
         Some(ty)
@@ -333,7 +354,7 @@ impl GotoTarget<'_> {
         model: &SemanticModel,
     ) -> Option<String> {
         if let GotoTarget::Call { call, .. } = self {
-            call_type_simplified_by_overloads(model.db(), model, call)
+            call_type_simplified_by_overloads(model, call)
         } else {
             None
         }
@@ -353,84 +374,56 @@ impl GotoTarget<'_> {
     /// as just returning a raw `NavigationTarget`.
     pub(crate) fn get_definition_targets<'db>(
         &self,
-        file: ruff_db::files::File,
-        db: &'db dyn crate::Db,
+        model: &SemanticModel<'db>,
         alias_resolution: ImportAliasResolution,
-    ) -> Option<DefinitionsOrTargets<'db>> {
-        use crate::NavigationTarget;
-
-        match self {
-            GotoTarget::Expression(expression) => definitions_for_expression(db, file, expression)
-                .map(DefinitionsOrTargets::Definitions),
-
+    ) -> Option<Definitions<'db>> {
+        let definitions = match self {
+            GotoTarget::Expression(expression) => definitions_for_expression(model, *expression),
             // For already-defined symbols, they are their own definitions
-            GotoTarget::FunctionDef(function) => {
-                let model = SemanticModel::new(db, file);
-                Some(DefinitionsOrTargets::Definitions(vec![
-                    ResolvedDefinition::Definition(function.definition(&model)),
-                ]))
-            }
+            GotoTarget::FunctionDef(function) => Some(vec![ResolvedDefinition::Definition(
+                function.definition(model),
+            )]),
 
-            GotoTarget::ClassDef(class) => {
-                let model = SemanticModel::new(db, file);
-                Some(DefinitionsOrTargets::Definitions(vec![
-                    ResolvedDefinition::Definition(class.definition(&model)),
-                ]))
-            }
+            GotoTarget::ClassDef(class) => Some(vec![ResolvedDefinition::Definition(
+                class.definition(model),
+            )]),
 
-            GotoTarget::Parameter(parameter) => {
-                let model = SemanticModel::new(db, file);
-                Some(DefinitionsOrTargets::Definitions(vec![
-                    ResolvedDefinition::Definition(parameter.definition(&model)),
-                ]))
-            }
+            GotoTarget::Parameter(parameter) => Some(vec![ResolvedDefinition::Definition(
+                parameter.definition(model),
+            )]),
 
             // For import aliases (offset within 'y' or 'z' in "from x import y as z")
             GotoTarget::ImportSymbolAlias {
                 alias, import_from, ..
             } => {
                 let symbol_name = alias.name.as_str();
-                Some(DefinitionsOrTargets::Definitions(
-                    definitions_for_imported_symbol(
-                        db,
-                        file,
-                        import_from,
-                        symbol_name,
-                        alias_resolution,
-                    ),
+                Some(definitions_for_imported_symbol(
+                    model,
+                    import_from,
+                    symbol_name,
+                    alias_resolution,
                 ))
             }
 
             GotoTarget::ImportModuleComponent {
                 module_name,
                 component_index,
+                level,
                 ..
             } => {
-                // Handle both `import foo.bar` and `from foo.bar import baz` where offset is within module component
-                let components: Vec<&str> = module_name.split('.').collect();
-
-                // Build the module name up to and including the component containing the offset
-                let target_module_name = components[..=*component_index].join(".");
-
-                // Try to resolve the module
-                definitions_for_module(db, &target_module_name)
+                // We don't currently support hovering the bare `.` so there is always a name
+                let module = import_name(module_name, *component_index);
+                definitions_for_module(model, Some(module), *level)
             }
 
             // Handle import aliases (offset within 'z' in "import x.y as z")
             GotoTarget::ImportModuleAlias { alias } => {
                 if alias_resolution == ImportAliasResolution::ResolveAliases {
-                    let full_module_name = alias.name.as_str();
-                    // Try to resolve the module
-                    definitions_for_module(db, full_module_name)
+                    definitions_for_module(model, Some(alias.name.as_str()), 0)
                 } else {
-                    let alias_range = alias.asname.as_ref().unwrap().range;
-                    Some(DefinitionsOrTargets::Targets(
-                        crate::NavigationTargets::single(NavigationTarget {
-                            file,
-                            focus_range: alias_range,
-                            full_range: alias.range(),
-                        }),
-                    ))
+                    alias.asname.as_ref().map(|name| {
+                        definitions_for_name(model, name.as_str(), AnyNodeRef::Identifier(name))
+                    })
                 }
             }
 
@@ -438,77 +431,128 @@ impl GotoTarget<'_> {
             GotoTarget::KeywordArgument {
                 keyword,
                 call_expression,
-            } => Some(DefinitionsOrTargets::Definitions(
-                definitions_for_keyword_argument(db, file, keyword, call_expression),
+            } => Some(definitions_for_keyword_argument(
+                model,
+                keyword,
+                call_expression,
             )),
 
             // For exception variables, they are their own definitions (like parameters)
             GotoTarget::ExceptVariable(except_handler) => {
-                let model = SemanticModel::new(db, file);
-                Some(DefinitionsOrTargets::Definitions(vec![
-                    ResolvedDefinition::Definition(except_handler.definition(&model)),
-                ]))
+                Some(vec![ResolvedDefinition::Definition(
+                    except_handler.definition(model),
+                )])
             }
 
-            // For pattern match rest variables, they are their own definitions
+            // Patterns are glorified assignments but we have to look them up by ident
+            // because they're not expressions
             GotoTarget::PatternMatchRest(pattern_mapping) => {
-                if let Some(rest_name) = &pattern_mapping.rest {
-                    let range = rest_name.range;
-                    Some(DefinitionsOrTargets::Targets(
-                        crate::NavigationTargets::single(NavigationTarget::new(file, range)),
-                    ))
-                } else {
-                    None
-                }
+                pattern_mapping.rest.as_ref().map(|name| {
+                    definitions_for_name(model, name.as_str(), AnyNodeRef::Identifier(name))
+                })
             }
 
-            // For pattern match as names, they are their own definitions
-            GotoTarget::PatternMatchAsName(pattern_as) => {
-                if let Some(name) = &pattern_as.name {
-                    let range = name.range;
-                    Some(DefinitionsOrTargets::Targets(
-                        crate::NavigationTargets::single(NavigationTarget::new(file, range)),
-                    ))
-                } else {
-                    None
-                }
+            GotoTarget::PatternMatchAsName(pattern_as) => pattern_as.name.as_ref().map(|name| {
+                definitions_for_name(model, name.as_str(), AnyNodeRef::Identifier(name))
+            }),
+
+            GotoTarget::PatternKeywordArgument(pattern_keyword) => {
+                let name = &pattern_keyword.attr;
+                Some(definitions_for_name(
+                    model,
+                    name.as_str(),
+                    AnyNodeRef::Identifier(name),
+                ))
+            }
+
+            GotoTarget::PatternMatchStarName(pattern_star) => {
+                pattern_star.name.as_ref().map(|name| {
+                    definitions_for_name(model, name.as_str(), AnyNodeRef::Identifier(name))
+                })
             }
 
             // For callables, both the definition of the callable and the actual function impl are relevant.
             //
             // Prefer the function impl over the callable so that its docstrings win if defined.
             GotoTarget::Call { callable, call } => {
-                let mut definitions = definitions_for_callable(db, file, call);
+                let mut definitions = definitions_for_callable(model, call);
                 let expr_definitions =
-                    definitions_for_expression(db, file, callable).unwrap_or_default();
+                    definitions_for_expression(model, *callable).unwrap_or_default();
                 definitions.extend(expr_definitions);
 
                 if definitions.is_empty() {
                     None
                 } else {
-                    Some(DefinitionsOrTargets::Definitions(definitions))
+                    Some(definitions)
                 }
             }
 
             GotoTarget::BinOp { expression, .. } => {
-                let model = SemanticModel::new(db, file);
-
                 let (definitions, _) =
-                    ty_python_semantic::definitions_for_bin_op(db, &model, expression)?;
+                    ty_python_semantic::definitions_for_bin_op(model, expression)?;
 
-                Some(DefinitionsOrTargets::Definitions(definitions))
+                Some(definitions)
             }
 
             GotoTarget::UnaryOp { expression, .. } => {
-                let model = SemanticModel::new(db, file);
                 let (definitions, _) =
-                    ty_python_semantic::definitions_for_unary_op(db, &model, expression)?;
+                    ty_python_semantic::definitions_for_unary_op(model, expression)?;
 
-                Some(DefinitionsOrTargets::Definitions(definitions))
+                Some(definitions)
             }
 
-            _ => None,
-        }
+            // String annotations sub-expressions require us to recurse into the sub-AST
+            GotoTarget::StringAnnotationSubexpr {
+                string_expr,
+                subrange,
+                ..
+            } => {
+                let (subast, submodel) = model.enter_string_annotation(string_expr)?;
+                let subexpr = covering_node(subast.syntax().into(), *subrange)
+                    .node()
+                    .as_expr_ref()?;
+                definitions_for_expression(&submodel, subexpr)
+            }
+
+            // nonlocal and global are essentially loads, but again they're statements,
+            // so we need to look them up by ident
+            GotoTarget::NonLocal { identifier } | GotoTarget::Globals { identifier } => {
+                Some(definitions_for_name(
+                    model,
+                    identifier.as_str(),
+                    AnyNodeRef::Identifier(identifier),
+                ))
+            }
+
+            // These are declarations of sorts, but they're stmts and not exprs, so look up by ident.
+            GotoTarget::TypeParamTypeVarName(type_var) => {
+                let name = &type_var.name;
+                Some(definitions_for_name(
+                    model,
+                    name.as_str(),
+                    AnyNodeRef::Identifier(name),
+                ))
+            }
+
+            GotoTarget::TypeParamParamSpecName(name) => {
+                let name = &name.name;
+                Some(definitions_for_name(
+                    model,
+                    name.as_str(),
+                    AnyNodeRef::Identifier(name),
+                ))
+            }
+
+            GotoTarget::TypeParamTypeVarTupleName(name) => {
+                let name = &name.name;
+                Some(definitions_for_name(
+                    model,
+                    name.as_str(),
+                    AnyNodeRef::Identifier(name),
+                ))
+            }
+        };
+        definitions.map(Definitions)
     }
 
     /// Returns the text representation of this goto target.
@@ -526,6 +570,7 @@ impl GotoTarget<'_> {
                 ast::ExprRef::Attribute(attr) => Some(Cow::Borrowed(attr.attr.as_str())),
                 _ => None,
             },
+            GotoTarget::StringAnnotationSubexpr { name, .. } => name.as_deref().map(Cow::Borrowed),
             GotoTarget::FunctionDef(function) => Some(Cow::Borrowed(function.name.as_str())),
             GotoTarget::ClassDef(class) => Some(Cow::Borrowed(class.name.as_str())),
             GotoTarget::Parameter(parameter) => Some(Cow::Borrowed(parameter.name.as_str())),
@@ -586,6 +631,7 @@ impl GotoTarget<'_> {
 
     /// Creates a `GotoTarget` from a `CoveringNode` and an offset within the node
     pub(crate) fn from_covering_node<'a>(
+        model: &SemanticModel,
         covering_node: &crate::find_node::CoveringNode<'a>,
         offset: TextSize,
         tokens: &Tokens,
@@ -632,6 +678,7 @@ impl GotoTarget<'_> {
                                 {
                                     return Some(GotoTarget::ImportModuleComponent {
                                         module_name: full_name.to_string(),
+                                        level: 0,
                                         component_index,
                                         component_range,
                                     });
@@ -672,14 +719,12 @@ impl GotoTarget<'_> {
                     // Handle offset within module name in from import statements
                     if let Some(module_expr) = &from.module {
                         let full_module_name = module_expr.to_string();
-
-                        if let Some((component_index, component_range)) = find_module_component(
-                            &full_module_name,
-                            module_expr.range.start(),
-                            offset,
-                        ) {
+                        if let Some((component_index, component_range)) =
+                            find_module_component(&full_module_name, module_expr.start(), offset)
+                        {
                             return Some(GotoTarget::ImportModuleComponent {
                                 module_name: full_module_name,
+                                level: from.level,
                                 component_index,
                                 component_range,
                             });
@@ -786,6 +831,31 @@ impl GotoTarget<'_> {
                 Some(GotoTarget::Expression(unary.into()))
             }
 
+            node @ AnyNodeRef::ExprStringLiteral(string_expr) => {
+                // Check if we've clicked on a sub-GotoTarget inside a string annotation's sub-AST
+                if let Some((subast, submodel)) = model.enter_string_annotation(string_expr)
+                    && let Some(GotoTarget::Expression(subexpr)) = find_goto_target_impl(
+                        &submodel,
+                        subast.tokens(),
+                        subast.syntax().into(),
+                        offset,
+                    )
+                {
+                    let name = match subexpr {
+                        ast::ExprRef::Name(name) => Some(name.id.to_string()),
+                        ast::ExprRef::Attribute(attr) => Some(attr.attr.to_string()),
+                        _ => None,
+                    };
+                    Some(GotoTarget::StringAnnotationSubexpr {
+                        string_expr,
+                        subrange: subexpr.range(),
+                        name,
+                    })
+                } else {
+                    node.as_expr_ref().map(GotoTarget::Expression)
+                }
+            }
+
             node => {
                 // Check if this is seemingly a callable being invoked (the `x` in `x(...)`)
                 let parent = covering_node.parent();
@@ -821,6 +891,7 @@ impl Ranged for GotoTarget<'_> {
             GotoTarget::ImportModuleComponent {
                 component_range, ..
             } => *component_range,
+            GotoTarget::StringAnnotationSubexpr { subrange, .. } => *subrange,
             GotoTarget::ImportModuleAlias { alias } => alias.asname.as_ref().unwrap().range,
             GotoTarget::ExceptVariable(except) => except.name.as_ref().unwrap().range,
             GotoTarget::KeywordArgument { keyword, .. } => keyword.arg.as_ref().unwrap().range,
@@ -840,9 +911,9 @@ impl Ranged for GotoTarget<'_> {
 }
 
 /// Converts a collection of `ResolvedDefinition` items into `NavigationTarget` items.
-fn convert_resolved_definitions_to_targets(
-    db: &dyn crate::Db,
-    definitions: Vec<ty_python_semantic::ResolvedDefinition<'_>>,
+fn convert_resolved_definitions_to_targets<'db>(
+    db: &'db dyn ty_python_semantic::Db,
+    definitions: Vec<ty_python_semantic::ResolvedDefinition<'db>>,
 ) -> Vec<crate::NavigationTarget> {
     definitions
         .into_iter()
@@ -876,27 +947,28 @@ fn convert_resolved_definitions_to_targets(
 
 /// Shared helper to get definitions for an expr (that is presumably a name/attr)
 fn definitions_for_expression<'db>(
-    db: &'db dyn crate::Db,
-    file: ruff_db::files::File,
-    expression: &ruff_python_ast::ExprRef<'_>,
+    model: &SemanticModel<'db>,
+    expression: ruff_python_ast::ExprRef<'_>,
 ) -> Option<Vec<ResolvedDefinition<'db>>> {
     match expression {
-        ast::ExprRef::Name(name) => Some(definitions_for_name(db, file, name)),
+        ast::ExprRef::Name(name) => Some(definitions_for_name(
+            model,
+            name.id.as_str(),
+            expression.into(),
+        )),
         ast::ExprRef::Attribute(attribute) => Some(ty_python_semantic::definitions_for_attribute(
-            db, file, attribute,
+            model, attribute,
         )),
         _ => None,
     }
 }
 
 fn definitions_for_callable<'db>(
-    db: &'db dyn crate::Db,
-    file: ruff_db::files::File,
+    model: &SemanticModel<'db>,
     call: &ast::ExprCall,
 ) -> Vec<ResolvedDefinition<'db>> {
-    let model = SemanticModel::new(db, file);
     // Attempt to refine to a specific call
-    let signature_info = call_signature_details(db, &model, call);
+    let signature_info = call_signature_details(model, call);
     signature_info
         .into_iter()
         .filter_map(|signature| signature.definition.map(ResolvedDefinition::Definition))
@@ -905,7 +977,7 @@ fn definitions_for_callable<'db>(
 
 /// Shared helper to map and convert resolved definitions into navigation targets.
 fn definitions_to_navigation_targets<'db>(
-    db: &dyn crate::Db,
+    db: &dyn ty_python_semantic::Db,
     stub_mapper: Option<&StubMapper<'db>>,
     mut definitions: Vec<ty_python_semantic::ResolvedDefinition<'db>>,
 ) -> Option<crate::NavigationTargets> {
@@ -920,12 +992,21 @@ fn definitions_to_navigation_targets<'db>(
     }
 }
 
-pub(crate) fn find_goto_target(
-    parsed: &ParsedModuleRef,
+pub(crate) fn find_goto_target<'a>(
+    model: &'a SemanticModel,
+    parsed: &'a ParsedModuleRef,
     offset: TextSize,
-) -> Option<GotoTarget<'_>> {
-    let token = parsed
-        .tokens()
+) -> Option<GotoTarget<'a>> {
+    find_goto_target_impl(model, parsed.tokens(), parsed.syntax().into(), offset)
+}
+
+pub(crate) fn find_goto_target_impl<'a>(
+    model: &'a SemanticModel,
+    tokens: &'a Tokens,
+    syntax: AnyNodeRef<'a>,
+    offset: TextSize,
+) -> Option<GotoTarget<'a>> {
+    let token = tokens
         .at_offset(offset)
         .max_by_key(|token| match token.kind() {
             TokenKind::Name
@@ -946,30 +1027,24 @@ pub(crate) fn find_goto_target(
         return None;
     }
 
-    let covering_node = covering_node(parsed.syntax().into(), token.range())
-        .find_first(|node| node.is_identifier() || node.is_expression())
+    let covering_node = covering_node(syntax, token.range())
+        .find_first(|node| {
+            node.is_identifier() || node.is_expression() || node.is_stmt_import_from()
+        })
         .ok()?;
 
-    GotoTarget::from_covering_node(&covering_node, offset, parsed.tokens())
+    GotoTarget::from_covering_node(model, &covering_node, offset, tokens)
 }
 
 /// Helper function to resolve a module name and create a navigation target.
 fn definitions_for_module<'db>(
-    db: &'db dyn crate::Db,
-    module_name_str: &str,
-) -> Option<DefinitionsOrTargets<'db>> {
-    use ty_python_semantic::{ModuleName, resolve_module};
-
-    if let Some(module_name) = ModuleName::new(module_name_str) {
-        if let Some(resolved_module) = resolve_module(db, &module_name) {
-            if let Some(module_file) = resolved_module.file(db) {
-                return Some(DefinitionsOrTargets::Definitions(vec![
-                    ResolvedDefinition::Module(module_file),
-                ]));
-            }
-        }
-    }
-    None
+    model: &SemanticModel<'db>,
+    module: Option<&str>,
+    level: u32,
+) -> Option<Vec<ResolvedDefinition<'db>>> {
+    let module = model.resolve_module(module, level)?;
+    let file = module.file(model.db())?;
+    Some(vec![ResolvedDefinition::Module(file)])
 }
 
 /// Helper function to extract module component information from a dotted module name
@@ -983,9 +1058,7 @@ fn find_module_component(
 
     // Split the module name into components and find which one contains the offset
     let mut current_pos = 0;
-    let components: Vec<&str> = full_module_name.split('.').collect();
-
-    for (i, component) in components.iter().enumerate() {
+    for (i, component) in full_module_name.split('.').enumerate() {
         let component_start = current_pos;
         let component_end = current_pos + component.len();
 
@@ -1004,3 +1077,16 @@ fn find_module_component(
 
     None
 }
+
+/// Helper to get the module name up to the given component index
+fn import_name(module_name: &str, component_index: usize) -> &str {
+    // We want everything to the left of the nth `.`
+    // If there's no nth `.` then we want the whole thing.
+    let idx = module_name
+        .match_indices('.')
+        .nth(component_index)
+        .map(|(idx, _)| idx)
+        .unwrap_or(module_name.len());
+
+    &module_name[..idx]
+}
