@@ -88,6 +88,10 @@ impl SemanticDb for Db {
     fn lint_registry(&self) -> &LintRegistry {
         default_lint_registry()
     }
+
+    fn verbose(&self) -> bool {
+        false
+    }
 }
 
 #[salsa::db]
